@@ -479,6 +479,7 @@ struct ImageWorker {
   void copyImageData(uint32_t idx, std::vector<VkSemaphore> sems);
 };
 struct PrimusSwapchain{
+  int max_fps = 0;
   InstanceInfo &myInstance;
   std::chrono::steady_clock::time_point lastPresent = std::chrono::steady_clock::now();
   VkDevice device;
@@ -495,6 +496,9 @@ struct PrimusSwapchain{
   std::vector<std::unique_ptr<std::thread>> threads;
 
   std::shared_ptr<CreateOtherDevice> cod;
+
+  bool suppress_suboptimal = false;
+
   PrimusSwapchain(PrimusSwapchain &) = delete;
   PrimusSwapchain(InstanceInfo &myInstance, VkDevice device, VkDevice display_device, VkSwapchainKHR backend, const VkSwapchainCreateInfoKHR *pCreateInfo, std::shared_ptr<CreateOtherDevice> &cod):
     myInstance(myInstance), device(device), display_device(display_device), backend(backend), cod(cod){
@@ -506,6 +510,13 @@ struct PrimusSwapchain{
 
     instance_dispatch[GetKey(myInstance.instance)].GetPhysicalDeviceSurfaceCapabilitiesKHR(myInstance.display, pCreateInfo->surface, &surfaceCapabilities);
     TRACE("Min Images: " << surfaceCapabilities.minImageCount);
+    char *max_fps_env = getenv("PRIMUS_VK_MAX_FPS");
+    if(max_fps_env != nullptr){
+      max_fps = std::stoi(std::string{max_fps_env});
+    }
+    if(getenv("PVK_SUPPRESS_SUBOPTIMAL")){
+      suppress_suboptimal = true;
+    }
 
     uint32_t image_count;
     device_dispatch[GetKey(display_device)].GetSwapchainImagesKHR(display_device, backend, &image_count, nullptr);
@@ -968,6 +979,10 @@ VkResult VKAPI_CALL PrimusVK_AcquireNextImage2KHR(VkDevice device, const VkAcqui
   device_dispatch[GetKey(ch->render_queue)].QueueSubmit(ch->render_queue, 1, &qsi, pAcquireInfo->fence);
   TRACE_PROFILING_EVENT(*pImageIndex, "Acquire done");
 
+  if(ch->suppress_suboptimal && res == VK_SUBOPTIMAL_KHR){
+    res = VK_SUCCESS;
+  }
+
   return res;
 }
 VkResult VKAPI_CALL PrimusVK_AcquireNextImageKHR(VkDevice device, VkSwapchainKHR swapchain, uint64_t timeout, VkSemaphore semaphore, VkFence fence, uint32_t* pImageIndex) {
@@ -1194,6 +1209,9 @@ void PrimusSwapchain::present(const QueueItem &workItem){
       has_work.wait(lock, [this,&workItem](){return &workItem == &in_progress.front();});
       TRACE_PROFILING_EVENT(index, "submitting");
       VkResult res = device_dispatch[GetKey(display_device)].QueuePresentKHR(display_queue, &p2);
+      if(suppress_suboptimal && res == VK_SUBOPTIMAL_KHR){
+	res = VK_SUCCESS;
+      }
       if(res != VK_SUCCESS && res != VK_SUBOPTIMAL_KHR) {
 	TRACE("ERROR, Queue Present failed: " << res << "\n");
       }
@@ -1225,7 +1243,7 @@ VkResult VKAPI_CALL PrimusVK_QueueSubmit(VkQueue queue, uint32_t submitCount,
 
 VkResult VKAPI_CALL PrimusVK_QueuePresentKHR(VkQueue queue, const VkPresentInfoKHR* pPresentInfo) {
   scoped_lock lock(*device_instance_info[GetKey(queue)]->renderQueueMutex);
-  const auto start = std::chrono::steady_clock::now();
+  auto start = std::chrono::steady_clock::now();
   if(pPresentInfo->swapchainCount != 1){
     TRACE("Warning, presenting with multiple swapchains not implemented, ignoring");
   }
@@ -1234,7 +1252,15 @@ VkResult VKAPI_CALL PrimusVK_QueuePresentKHR(VkQueue queue, const VkPresentInfoK
   double secs = std::chrono::duration_cast<std::chrono::duration<double>>(start - ch->lastPresent).count();
   TRACE_PROFILING_EVENT(pPresentInfo->pImageIndices[0], "QueuePresent");
   TRACE_PROFILING(" === Time between VkQueuePresents: " << secs << " -> " << 1/secs << " FPS");
-  ch->lastPresent = start;
+  if(ch->max_fps != 0) {
+    const auto timeBetweenFrame = std::chrono::milliseconds(1000) / ch->max_fps;
+    const auto toSleep = ch->lastPresent + timeBetweenFrame - start;
+    if(toSleep > std::chrono::seconds(0)){
+      std::this_thread::sleep_for(toSleep);
+      start = std::chrono::steady_clock::now();
+    }
+    ch->lastPresent = start;
+  }
 
   ch->queue(queue, pPresentInfo);
 
@@ -1404,7 +1430,7 @@ VkResult VKAPI_CALL PrimusVK_GetRandROutputDisplayEXT(
     Display*                                    dpy,
     RROutput                                    rrOutput,
     VkDisplayKHR*                               pDisplay){
-  TRACE("Surpressing GetRandROutputDisplay");
+  TRACE("Suppressing GetRandROutputDisplay");
   *pDisplay = VK_NULL_HANDLE;
   return VK_SUCCESS;
 }
